/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/config/conf_parse.y,v 1.2 2002/04/02 02:07:05 koni Exp $ 
  $Author: koni $ $Date: 2002/04/02 02:07:05 $ $Name: slan-release-0-04 $
  $Revision: 1.2 $

*/
   
/* Configuration parser for the SLAN server. Testing version */

%{
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_GMP_H
#include <gmp.h>
#else
#include "slan_gmp3.h"
#endif
#include "slan_config.h"
  extern slan_conf_t *slan_global_config;
 static void yyerror(char *s);
 char *get_yytext(void);
 int yylex(void);
 static void load_private_key(char *filename);
%}

%union {
       int itype;
       char * string;
}

%token <string> HEXSTRING
%token <string> STRING
%token <string> IPADDR
%token <string> HOSTNAME
%token <itype>    INTEGER
%token YES
%token NO

%token CONTROL_PORT
%token TRANSPORT_PORT

%token SERVICE_IDENTIFIER
%token NODE_IDENTIFIER
%token RSA_LABEL
%token RSA_MODULUS
%token RSA_PUBLIC
%token RSA_PRIVATE
%token DH_PRIME
%token DH_GENERATOR

%token ASSIGN_IPS
%token IPMASQ
%token SLAN_IPADDRESS
%token BROADCAST
%token NETMASK
%token ROUTER
%token NAMESERVER
%token FLEX_ERROR
%token RSAKEY
%token OPEN_BRACE
%token CLOSE_BRACE
%token DH
%token SERVER_ADDRESS
%token ENDL

%type <itype> integer
%type <string> string
%type <string> hexstring
%%
input:  /* Basis (empty input) */
	| input line
	;

line:	parameter
	;

parameter:	service_identifier ENDL
		| node_identifier ENDL
		| control_port ENDL
		| transport_port ENDL
		| rsakey 
		| dh 
		| assign_ips ENDL
                | ipmasq ENDL
		| slan_ipaddress ENDL
		| router ENDL
		| broadcast ENDL
		| netmask ENDL
		| nameserver ENDL
		| server_address ENDL
		;

service_identifier: SERVICE_IDENTIFIER string { slan_global_config->service_identifier = $2; }
node_identifier:    NODE_IDENTIFIER string { slan_global_config->node_identifier = $2; }
control_port:	    CONTROL_PORT integer { if ($2 > 0 || $2 < 0xFFFF) slan_global_config->control_port = $2; }
transport_port:	    TRANSPORT_PORT integer { if ($2 > 0 || $2 < 0xFFFF) slan_global_config->transport_port = $2; }
rsakey:		    RSAKEY OPEN_BRACE rsa_items CLOSE_BRACE
rsa_items:	    
		    | rsa_items rsa_modulus ENDL
		    | rsa_items	rsa_public ENDL
		    | rsa_items rsa_private ENDL
		    | rsa_items rsa_label ENDL
		    ;

rsa_label:	    RSA_LABEL string { slan_global_config->rsakey->label = $2; }
rsa_modulus:	    RSA_MODULUS HEXSTRING { slan_global_config->rsakey->modulus = strdup(get_yytext()); mpz_set_str(slan_global_config->rsakey->modulus_mpz, get_yytext(), 0); }
rsa_public:	    RSA_PUBLIC  HEXSTRING { slan_global_config->rsakey->public = strdup(get_yytext());  mpz_set_str(slan_global_config->rsakey->public_mpz, get_yytext(), 0); }
rsa_private:	    RSA_PRIVATE string { /*slan_global_config->rsakey->private
					   = strdup(get_yytext());
					   mpz_set_str(slan_global_config->rsakey->private_mpz, get_yytext(), 16);*/ load_private_key($2); /*free($2);*/ }
dh:		    DH	OPEN_BRACE dh_items CLOSE_BRACE
dh_items:	    
		    | dh_items dh_generator ENDL
		    | dh_items dh_prime ENDL
		    ;

dh_prime:	    DH_PRIME	hexstring   { slan_global_config->dh->prime = $2; }
		    

dh_generator:	    DH_GENERATOR hexstring { slan_global_config->dh->gen = $2; }
		    

assign_ips:	    ASSIGN_IPS YES { slan_global_config->assign_ips = 1; };
		    | ASSIGN_IPS NO { slan_global_config->assign_ips = 0; };
ipmasq:             IPMASQ YES { slan_global_config->ipmasq = 1; };
                    | IPMASQ NO { slan_global_config->ipmasq = 0; }; 
slan_ipaddress:	    SLAN_IPADDRESS IPADDR { slan_global_config->slan_ipaddress = strdup(get_yytext()); }
router:		    ROUTER IPADDR  {slan_global_config->router = strdup(get_yytext()); }
broadcast:	    BROADCAST IPADDR {slan_global_config->broadcast = strdup(get_yytext()); }
netmask:	    NETMASK IPADDR {slan_global_config->netmask = strdup(get_yytext()); }
nameserver:	    NAMESERVER IPADDR { slan_global_config->nameserver1 = strdup(get_yytext()); }
integer:            INTEGER { $$ = atoi(get_yytext()); }

server_address:	    SERVER_ADDRESS IPADDR { slan_global_config->server_address = strdup(get_yytext()); }
		    | SERVER_ADDRESS HOSTNAME { slan_global_config->server_address = strdup(get_yytext()); }		

string:		    STRING { $$ = strdup(get_yytext()+1); $$[strlen($$)-1]=0; };
hexstring:          HEXSTRING { $$ = strdup(get_yytext());  }
%%


extern int yylineno;
static void yyerror(char *s) {
  fprintf(stderr,"Error reading configuration:\n");
  fprintf(stderr,"%s at %s on line %d\n",s,get_yytext(),yylineno);
}

/* This is really large, the input is expected to be hexadecimal ascii! Thus
   anything larger than this is certainly not a private key. Anything after 
   the first line will not be interpreted. */
#define MAX_PRIVATE_KEYSIZE (4096)
static void load_private_key(char *filename) {
  FILE *f;
  unsigned char *p;
  int length, i;

  if (slan_global_config->server == 0) {
    fprintf(stderr,"Warning: client configuration should not specify a private"
	    " key file.\n");
    fprintf(stderr,"Skipping private key option.\n");
    return;
  }

  f = fopen(filename, "r");
  if (f == NULL) {
    fprintf(stderr,"Cannot open private key file \"%s\" specified in "
	    "configuration file.\n",filename);
    return;
  }

  p = malloc(MAX_PRIVATE_KEYSIZE);
  fgets(p, MAX_PRIVATE_KEYSIZE, f);
  length = strlen(p) - 1;
  if (p[length] != '\n') {
    fprintf(stderr,"Error: Private key data in file \"%s\" is too long or "
	    "not in proper format.\n", filename);
    free(p);
    fclose(f);
    return;
  } else {
    p[length] = '\0';
  }

  for(i=0;i<length;i++) {
    if ((p[i] >= '0' && p[i] <='9') ||
	(p[i] >= 'A' && p[i] <='F') ||
	(p[i] >= 'a' && p[i] <= 'f')) continue;
    else {
      fprintf(stderr,"Error: Private key data in file \"%s\" is "
	      "not in proper format. Hexadecimal text expected.\n", filename);
      free(p);
      fclose(f);
      return;
    }
  }

  slan_global_config->rsakey->private = strdup(p);
  mpz_set_str(slan_global_config->rsakey->private_mpz, p, 16);
  
  free(p);
  fclose(f);

}

