/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/hash_algorithms/hmac_sha1.c,v 1.1.1.1 1999/12/09 01:16:35 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:35 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#ifndef WIN32
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#else
#include "slan_windows.h"
#endif

#include "sha.h"
#include "mac.h"

#include "slan_types.h"
#include "slan_alloc.h"

static int hmac_setkey(mac_t *us, unsigned char *key, int key_length) {

  if (us->key != NULL) {
    us->key = slan_realloc(us->key, key_length);
  } else {
    us->key = slan_malloc(key_length);
  }

  if (us->key == NULL) return 1;

  /* HMAC-SHA1 specifies 64 byte key maximum -- supposed to run the hash
     algorithm on the key if necessary to reduce the key size, but that seems
     somewhat assinine if the key can be assumed to have sufficient entropy */
  if (key_length > 64) us->key_length = 64;
  else us->key_length = key_length;

  memcpy(us->key, key, us->key_length);
  us->key_length = key_length;
  
  return 0;
}

static void hmac_message_nocopy_sign(mac_t *us, unsigned char *message, int
				    message_length, unsigned char *digest) {
  unsigned char second_message[84];
  unsigned char temp_digest[20];
  unsigned char *first_message;
  int i;

  first_message = message - 64;
  
  memset(first_message, 0x36, 64);
  memset(second_message, 0x5c, 64);

  for(i=0;i<us->key_length;i++) {
    first_message[i] ^= us->key[i];
    second_message[i] ^= us->key[i];
  }

  sha_digest(first_message, message_length + 64, second_message + 64);

  sha_digest(second_message, 84, temp_digest);

  memcpy(digest, temp_digest + 8, 12);
}

static int hmac_message_sign(mac_t *us, unsigned char *message, int
			     message_length, unsigned char *digest) {
  unsigned char *message_copy;
  
  message_copy = slan_malloc(message_length + 64);
  if (message_copy == NULL) return 1;
  memcpy(message_copy + 64, message, message_length);

  hmac_message_nocopy_sign(us, message_copy + 64, message_length, digest);
  slan_memfree(message_copy);
  return 1;
}

static int hmac_close(mac_t *us) {
  int i;

  /* Explicitly forget key if its no longer needed */
  for(i=0;i<us->key_length;i++) us->key[i]=0x0; 

  slan_memfree(us->key);
  slan_memfree(us);
  return 0;
}

mac_t *get_hmac_sha1(void) {
  mac_t *us;
  
  us = slan_calloc(1,sizeof(mac_t));
  if (us == NULL) return NULL;

  us->message_sign = hmac_message_sign;
  us->message_nocopy_sign = hmac_message_nocopy_sign;
  us->header_room = 64;
  us->trailer_room = 0;
  us->internal_data = NULL;
  us->close = hmac_close;
  us->set_key = hmac_setkey;
  us->hash_size = 12;
  us->key = NULL;

  return us;
}

