/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/include/mac.h,v 1.1.1.1 1999/12/09 01:16:35 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:35 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#ifndef _SLAN_MAC_H
#define _SLAN_MAC_H

typedef struct mac_struct {
  unsigned char *key;
  int key_length;
  /* This function _copies_ the key for the MAC into this structure. Returns
     non-zero if memory allocation failed */
  int (*set_key)(struct mac_struct *, unsigned char *, int);
  /* This function computes the Message Authentication Code and returns it in
     the last unsigned char * pointer. It may need to produce a duplicate of
     the message to compute the MAC, depending on what the actual MAC code
     is. It will return non-zero if making a duplicate copy of the message 
     failed. */
  int (*message_sign)(struct mac_struct *, unsigned char *, int, unsigned
		       char *);
  /* This next function does the same as the previous, but uses reserved space 
   at the end of the begining of the pointer passed as the message to compute
   the MAC, avoiding the overhead of copying the message. */
  void (*message_nocopy_sign)(struct mac_struct *, unsigned char *, int, 
			      unsigned char *);
  /* This is the amount of header or trailer room needed to do a nocopy 
     verify or sign */
  int header_room;
  int trailer_room;
  /* This is the size of the hash returned by compute_digest */
  int hash_size;
  /* This stores whatever the MAC might need in addition to key and 
     keylength */
  void *internal_data;
  /* Deallocates internal data and does any clean up work needed once it is
     known that this MAC is no longer needed */
  int (*close)(struct mac_struct *);
} mac_t;

/* Size of the maximum size Message Authentication Code, in bytes. Any MAC
   implementation which wants to use a larger MAC should not be supported, or
   this constant must be changed. This is because some parts of this code
   stack-allocate buffers to hold MACs, for efficiency, and allocate
   MAC_MAX_SIZE bytes. If the MAC is actually larger, hell will probably break 
   loose. */
#define MAC_MAX_SIZE (64)


#endif

