/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/include/slan_session.h,v 1.1.1.1 1999/12/09 01:16:37 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:37 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#ifndef SLAN_SESSION_H
#define SLAN_SESSION_H
#ifndef WIN32
#include <pthread.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#endif

#include "slan_types.h"

#include "sym_cipher.h"
#include "mac.h"
#include "proto_stack.h"

typedef struct slan_session_struct {
  /* Indentification and control information */
  u_int32_t id;
  tcp_socket_t *control_tcp;
  time_t last_control_keepalive;
  struct sockaddr_in *client_addr;
  u_int32_t ipaddr;
  unsigned char macaddr[6];
  u_int32_t control_seq_out;
  u_int32_t control_seq_in;
  pthread_mutex_t *lock;

  /* Accounting information */
  u_int32_t bytes_in;
  u_int32_t bytes_out;
  u_int32_t packets_in;
  u_int32_t packets_out;

  /* Session meta information */
  /* This is the session key: It will be set by the diffie-hellman key
     exchange. Later, when the transport layer is activated, it will be split
     into the symmetric cipher key and the MAC key. We will simply set the the 
     mackey pointer below to somewhere in the middle of the memory allocated
     to the key pointer, and adjust key_length and mackey_length 
     appropriately */
  int key_length;
  unsigned char *key;
  int mackey_length;
  unsigned char *mackey;
  time_t key_timestamp;
  int key_change;

  int keyhash_length;
  unsigned char *keyhash;

  int username_length;
  unsigned char *username;
  
  int password_length;
  unsigned char *password;

  /* Symmetric Cipher instance */
  cipher_t *cipher;

  /* Messsage Authentication Code instance */
  mac_t *mac;

  /* Transport layer */
  proto_stack_t *transport_top;
  proto_stack_t *transport_bottom;
  int header_save;
  u_int32_t sequence_down;
  u_int32_t sequence_up;
  int down_blocking;
  int up_blocking;
  int transport_activity;
  int udp_socket;
  int slan_socket;

  /* This is used to spawn a seperate thread to setup the session */
  pthread_t thread_obj;

  /* Linked list for chaining hash collisions */
  struct slan_session_struct *next;
} slan_session_t;
#endif

