/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/include/sym_cipher.h,v 1.1.1.1 1999/12/09 01:16:37 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:37 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#ifndef SYM_CIPHER_H
#define SYM_CIPHER_H
/* Methods:
   
   set_key:
   This function takes a char * to a new key in binary form, and a int 
   indicating the length of the key in _bytes_. It runs the cipher's
   keyschedule to prepare it to use the new key for encryption and
   decryption. 

   Encrypt:
   This function takes a char * to a buffer to be encrypted, and a int 
   indicating the length of the buffer in bytes. The buffer is
   encrypted in place.

   Decrypt:
   This function takes a char * to a buffer to be decrypted, and a int 
   indicating the length of the buffer in bytes. The buffer is
   decrypted in place.

   Close:
   This function takes care of any house cleaning that needs to be
   done and indicates the cipher will no longer be used. The cipher
   cannot be used unless open is called (and probably shouldn't be
   used until set_key is called, but this is responsibility of higher
   up)
   
   error:
   This function takes an error code (as returned by one of the other
   methods) a returns a pointer to a string describing the error.

   data:
   This is a pointer to memory that has been allocated for use by the
   block cipher's engine. It is set up when this struct is allocated.

   blocksize: in bytes.
   Self-explanitory.
*/

typedef struct symcipher {
  int (*set_key)(struct symcipher *, unsigned char *, int);
  void (*block_encrypt)(struct symcipher *, void *);
  void (*block_decrypt)(struct symcipher *, void *);
  void (*cbc_encrypt)(struct symcipher *, unsigned char *, int);
  void (*cbc_decrypt)(struct symcipher *, unsigned char *, int);
  int (*close)(struct symcipher *);
  char *(*error)(int);
  int blocksize;
  void *data; 
} cipher_t;
#if 0
#define MAX_BLOCKSIZE (128)

/* Cipher-block-chaining mode functions */
int sym_cbc_encrypt(cipher_t *cipher, unsigned char *buf, int
		    length);
int sym_cbc_decrypt(cipher_t *cipher, unsigned char *buf, int
		    length);
#endif
#endif

