/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/tests/hmac.c,v 1.1.1.1 1999/12/09 01:16:42 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:42 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#include <string.h>
#include <sys/types.h>
#include "sha.h"

static unsigned char ipad[64];
static unsigned char opad[64];

int set_key(unsigned char *hash_key, int hash_key_length) {
  int i;

  /* Even though RFC 2104 specifies that hash key lengths larger than
     the block size of the hash algorithm should "first be hashed by
     the algorithm" into a smaller key, we are just going to truncate
     to 64 bytes here. We assume (for now) that any hash algorithm we
     are going to used is going to have a 512 bit block length. */
  if (hash_key_length > 64) hash_key_length = 64;
  
  for(i=0;i<64;i++) {
    ipad[i] = 0x36;
    opad[i] = 0x5c;
  }

  for(i=0;i<hash_key_length;i++) {
    ipad[i] ^= hash_key[i];
    opad[i] ^= hash_key[i];
  }
  
  return 0;
}

void hmac_sha1_generate(unsigned char *message, int length,
				  unsigned char *digest) {
  /* Making an assumption here that we are not going to see messages
     larger than 10176 bytes. Otherwise, we need to increase the
     constant below or use malloc, but I want to avoid malloc()
     because it can be a costly call where as stack allocation costs
     nothing. What would be better is to have a zero-copy method
     here. */
  unsigned char message_copy[10240];

  memcpy(message_copy,ipad,64);
  memcpy(message_copy + 64, message, length);

  sha_digest(message_copy,length+64,digest);

  memcpy(message_copy,opad,64);
  memcpy(message_copy + 64, digest, 20);
  
  sha_digest(message_copy,84,digest);
}

      

