/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/tests/sym_ciphertest.c,v 1.1.1.1 1999/12/09 01:16:42 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:42 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "sym_cipher.h"
#include "blowfish/blowfish.h"

static cipher_t *cipher;
static int encrypt, decrypt, key_length;
static unsigned char key[56];

static void setup_cipher(unsigned char *key, int key_length) {

  cipher = get_blowfish();
  cipher->set_key(cipher, key, key_length);
}

static void extract_key(char *keydata) {
  int len;
  int i;
  unsigned char value;

  memset(key,0x0,56);

  len = strlen(keydata);
  if (len > 112) len = 112;
  key_length = len/2;
  
  for (i=0;i<len;i++) {
    if (keydata[i] >= '0' && keydata[i] <= '9') {
      value = (keydata[i] - '0');
    } else if (keydata[i] >= 'A' && keydata[i] <= 'F') {
      value = (keydata[i] - 'A') + 10;
    } else if (keydata[i] >= 'a' && keydata[i] <= 'f') {
      value = (keydata[i] - 'a') + 10;
    } else {
      fprintf(stderr,"Invalid hexadecimal digit %c, skipping.\n",
	      keydata[i]);
      continue;
    }
    value = value << ((i%2)?0:4);
    key[i/2] += value;
  }
}

  
/* Simple minded commandline parsing (because it sucks so much),
   unknown options are discarded. Lack of either -d or -e defaults to
   encrypt mode, multiple -d or -e uses the last one specified. */
static void parse_commandline(int argc, char *argv[]) {
  int i;
  char *arg;

  encrypt = 1;
  decrypt = 0;
  for(i=1;i<argc;i++) {
    arg = argv[i];
    if (strcmp(arg,"-d") == 0) {
      decrypt = 1;
      encrypt = 0;
    }
    if (strcmp(arg,"-e") == 0) {
      encrypt = 1;
      decrypt = 0;
    }
    if (strcmp(arg,"-k") == 0) {
      /* Next argument should be a key in hex with leading 0x*/
      if (i == (argc - 1)) {
	fprintf(stderr,"blowfish: Error, please specify a key in hex "
		"(with leading 0x) after -k\n");
	exit(-1);
      }
      if (strlen(argv[i+1])<=2 || strncmp(argv[i+1],"0x",2)) {
	fprintf(stderr,"blowfish: Error, please specify a key in hex "
		"(with leading 0x) after -k\n");
	exit(-1);
      }
      extract_key(argv[i+1] + 2);
    }
  }
}    

int main(int argc, char *argv[]) {
  int bytes_read, inputsize;
  unsigned char *inputbuffer;
  clock_t timestamp;

  parse_commandline(argc, argv);
  setup_cipher(key, key_length);

  bytes_read = 0;
  inputsize = 0;
  inputbuffer = NULL;

  inputbuffer = slan_calloc(1, 1024);
  timestamp = clock();
  if (encrypt) {
     do {
      bytes_read = fread(inputbuffer, 1, 1024, stdin);
      sym_cbc_encrypt(cipher, inputbuffer, bytes_read);
      fwrite(inputbuffer, 1, bytes_read, stdout);
      inputsize += bytes_read;
    } while(bytes_read == 1024);
  } else {
    do {
      bytes_read = fread(inputbuffer, 1, 1024, stdin);
      sym_cbc_decrypt(cipher, inputbuffer, bytes_read);
      fwrite(inputbuffer, 1, bytes_read, stdout);
      inputsize += bytes_read;
    } while(bytes_read == 1024);
  }
  timestamp = clock() - timestamp;

  fprintf(stderr,"Total time: %7.2f seconds.\n",timestamp/(float) 
	  (CLOCKS_PER_SEC));
  fprintf(stderr,"Speed: %7.2fM/sec.\n",
	  (inputsize/(1024.0*1024.0)/(timestamp/((float) CLOCKS_PER_SEC)))); 
  return 0;
}

