/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/transport/message_sign.c,v 1.1.1.1 1999/12/09 01:16:43 koni Exp $
  $Author: koni $ $Date: 1999/12/09 01:16:43 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/

#ifdef WIN32
#include "slan_windows.h"
#else
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#endif

#include "slan_types.h"
#include "slan_alloc.h"

#include "mac.h"
#include "proto_stack.h"
#include "slan_message.h"

/* Why bother with this? So we can "free" the data pointer for any protocol
   stack (if its not NULL) instead of using a destructor function */
typedef struct {
  mac_t *mac;
} mac_pointer_t;

static void message_sign_down(proto_stack_t *stack, slan_message_t *message) {
  mac_t *mac;

  mac = ((mac_pointer_t *) stack->data)->mac;

  slan_reserve(message, mac->header_room);
  mac->message_nocopy_sign(mac, message->data, message->length, 
			   message->data - mac->hash_size);
  
  slan_release(message, mac->header_room - mac->hash_size);
  slan_set(message);

  stack->next->down(stack->next, message);
}

static void message_sign_up(proto_stack_t *stack, slan_message_t *message) {
  /* I'm stack allocating MAC_MAX_SIZE bytes and hoping that is enough for any
     size MAC. To malloc the amount specified in mac->hash_size is some extra
     cycles that aren't really needed since a mac, by definition, is usually
     small. */
  unsigned char received_code[MAC_MAX_SIZE];
  unsigned char local_code[MAC_MAX_SIZE];
  mac_t *mac;

  mac = ((mac_pointer_t *) stack->data)->mac;

  /* Make a copy and pull the original MAC code */
  memcpy(received_code, message->data, mac->hash_size);
  slan_pop(message, mac->hash_size);

  slan_reserve(message, mac->header_room);
  mac->message_nocopy_sign(mac, message->data, message->length, local_code);

  if (memcmp(local_code, received_code, mac->hash_size)) {
#ifndef WIN32
    syslog(LOG_ERR,"Invalid (MAC failure) packet received. Dropped.\n");
#endif
    return;
  }

  slan_release(message, mac->header_room);
#ifdef DEBUG
  fprintf(stderr,"MAC verified okay.\n");
#endif
  stack->prev->up(stack->prev, message);
}

int message_sign_register(proto_stack_t *stack_entry, mac_t *mac) {

  stack_entry->up = message_sign_up;
  stack_entry->down = message_sign_down;
  stack_entry->header_reserve = mac->header_room;
  stack_entry->trailer_reserve = mac->trailer_room;
  stack_entry->data = (mac_t *) slan_malloc(sizeof(mac_pointer_t));
  ((mac_pointer_t *) stack_entry->data)->mac = mac;
  
  return 0;
}

