/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/transport/sequence.c,v 1.1.1.1 1999/12/09 01:16:43 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:43 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#ifndef WIN32
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <syslog.h>
#else
#include "slan_windows.h"
#endif

#include "slan_alloc.h"
#include "slan_types.h"

#include <proto_stack.h>
#include <slan_message.h>

#define LOWER_REJECT_LIMIT (20)

typedef struct {
  u_int32_t down_seq_number;
  u_int32_t up_seq_number;
} sequence_data_t;


static void sequence_down(proto_stack_t *stack, slan_message_t *message) {
  sequence_data_t *local;

  local = (sequence_data_t *) stack->data;
  slan_reserve(message, 4);
  *((u_int32_t *) message->header) = htonl(local->down_seq_number);
  
  local->down_seq_number++;

  /* Catch a sequence number rollover before it happens, and reset the 
     seqeunce number to 255 */
  if (local->down_seq_number > 0xFFFFFFF0) local->down_seq_number = 255;
  
  slan_set(message);

  stack->next->down(stack->next, message);
}

static void sequence_up(proto_stack_t *stack, slan_message_t *message) {
  u_int32_t packet_sequence;
  sequence_data_t *local;

  local = (sequence_data_t *) stack->data;
  packet_sequence = ntohl(*((u_int32_t *) message->header));

  /* Check for packets with sequence numbers less than we expect. This 
     is expected to happen only in case of rollover of the sequence
     numbers. We rollover not to zero but to 255 in order to check for 
     this. If our local sequence number is not near the rollover
     point, we reject any possiblity of rolling over */
  if (packet_sequence < (local->up_seq_number - LOWER_REJECT_LIMIT)) {
    /* Might be a rollover, check it out by seeing if we are possibly
       near the range for that to happen and if the sent sequence
       number is at or around 255, the rollover number. */
    if (local->up_seq_number > 0xFFFFF000 && 
	(packet_sequence>=255 && packet_sequence < (255+LOWER_REJECT_LIMIT))){ 
      ;
    } else {
      /* Out of seqeunce packet, possibly a replay attempt */
#ifndef WIN32
      syslog(LOG_ERR,"Dropped out of sequence packet.\n");
#endif
      return;
    }
  }


  /* Store currently found sequence number. Next packet received must
     have a sequence number within the range
     -LOWER_REJECT_LIMIT/inf*/
  local->up_seq_number = packet_sequence;
    
  /* Remove sequence header */
  slan_pop(message, 4);

  stack->prev->up(stack->prev, message);
}

int sequence_register(proto_stack_t *stack_entry) {
  sequence_data_t *local;

  stack_entry->up = sequence_up;
  stack_entry->down = sequence_down;
  stack_entry->header_reserve = 4;
  stack_entry->trailer_reserve = 0;
  local = slan_malloc(sizeof(sequence_data_t));
  if (local == NULL) return -1;
  stack_entry->data = local;

  local->up_seq_number = LOWER_REJECT_LIMIT + 1;
  local->down_seq_number = LOWER_REJECT_LIMIT + 1;

  return 0;
}

