/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/transport/server_udplink.c,v 1.1.1.1 1999/12/09 01:16:43 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:43 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
/*

  KONI Oct. 28-29, 2000
  Stage 1 client daemon process for "lightlink secure lan" project

  udp_link.c: This file implements functions to open, send, and receive data
              from a server via UDP.

  Oct. 29: This file split off because of header conflicts with the packet
           interface code.

*/

/* ---- Include files ----- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <syslog.h>
#include "proto_stack.h"
#include "slan_version.h"
#include "slan_alloc.h"

typedef struct {
  int fd;
  struct sockaddr_in client_addr;
  int throttle;
} server_udplink_t;


static void server_udplink_down(proto_stack_t *stack, slan_message_t
				*message) {
  server_udplink_t *local;
  int len;

  local = stack->data;

  len = sendto(local->fd,message->data,message->length, 0,
	       (struct sockaddr *) &local->client_addr, 
	       sizeof(struct sockaddr_in));
  if (len < message->length) {
    syslog(LOG_WARNING,"Error sending packet to client %s (%m)", inet_ntoa(local->client_addr.sin_addr));
  }
  
  stack->next->down(stack->next,message);
}

#if 0
static void server_udplink_up(proto_stack_t *stack, slan_message_t
			      *message) {
  server_udplink_t *local;
  int s_length, len, retval;
  struct sockaddr_in source_addr;
  fd_set rfds;
  struct timeval tv;

  local = stack->data;

  FD_ZERO(&rfds);
  FD_SET(local->fd, &rfds);
  
  tv.tv_sec = 0;
  tv.tv_usec = local->throttle / 2;

  retval = select(local->fd+1, &rfds, NULL, NULL, &tv);

  if (retval == -1 ) {
    /* Debugging stuff */
    perror("Error checking slan socket for data: ");
    slan_free(message);
    return;
  } else if (retval == 0) {
    local->throttle = (local->throttle + 1) % 50;
    slan_free(message);
    return;
  }
  
  local = stack->data;
  s_length = sizeof(source_addr);
  len = recvfrom(local->fd, message->data, message->maxspace, 0,
		 (struct sockaddr *) &source_addr, &s_length); 

  if (len < 0) {
    /* Debugging */
    perror("sland: error receiving data from client: ");
    slan_free(message);
    return;
  }

  message->length = len;

  stack->prev->up(stack->prev, message);
}
#endif

struct sockaddr_in *server_udplink_register(proto_stack_t *stack_entry, 
					    int udp_socket, int port) {
  server_udplink_t *local;

  stack_entry->up = NULL;//server_udplink_up;
  stack_entry->down = server_udplink_down;
  stack_entry->header_reserve = 0;
  stack_entry->trailer_reserve = 0;
  
  local = (server_udplink_t *) slan_calloc(1, sizeof(server_udplink_t));
  if (local == NULL) return NULL;

  local->fd = udp_socket;

  stack_entry->data = local;
  
  return &local->client_addr;
}

