/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/transport/session_id.c,v 1.1.1.1 1999/12/09 01:16:43 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:43 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
#ifndef WIN32
#include <stdlib.h>
#else
#include "slan_windows.h"
#endif

#include "slan_alloc.h"
#include "slan_types.h"

#include <proto_stack.h>
#include <slan_message.h>

typedef struct {
  u_int32_t session_id;
} session_id_t;

static void session_id_down(proto_stack_t *stack, slan_message_t *message) {
  session_id_t *local;

  local = (session_id_t *) stack->data;

  /* Prepend our session id number */
  slan_reserve(message, 4);
  *((u_int32_t *) message->header) = local->session_id;
  
  slan_set(message);

  stack->next->down(stack->next, message);
}

static void session_id_up(proto_stack_t *stack, slan_message_t *message) {
    
  /* Remove session_id header */
  slan_pop(message, 4);

  stack->prev->up(stack->prev, message);
}

int session_id_register(proto_stack_t *stack_entry) {
  session_id_t *local;

  stack_entry->up = session_id_up;
  stack_entry->down = session_id_down;
  stack_entry->header_reserve = 4;
  stack_entry->trailer_reserve = 0;
  local = slan_malloc(sizeof(session_id_t));

  if (local == NULL) return -1;
  stack_entry->data = local;

  return 0;
}

