/*
  Lightlink SLAN -- A Virtual Private Network System
  (c) 2001 Mark Wright (Koni), Lucas Madar, Art Matrix - Lightlink, Inc.  

  This program is free software; you can redistribute it and/or	
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  $Header: /home/cvsroot/stage6/transport/sym_cipher_cbc.c,v 1.1.1.1 1999/12/09 01:16:43 koni Exp $ 
  $Author: koni $ $Date: 1999/12/09 01:16:43 $ $Name: slan-release-0-04 $
  $Revision: 1.1.1.1 $

*/
   
/* Koni May 2001
   
   This file used to (back in the day) implement cipher-block-chaining mode
   generically for any underlying cipher -- but was found to be inadequate
   because of endianness problems resulting from how the underlying cipher
   actually worked. Thus, the cbc routines are now being built with the cipher
   implementation in symmetric ciphers. This just wraps it into the transport
   layer. */


#ifndef WIN32
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#else
#include "slan_windows.h"
#endif

#include "slan_types.h"

#include "sym_cipher.h"
#include "proto_stack.h"

/* Why bother with this? So we can deallocate the transport stack without
   needing destructor functions. If the stack->data pointer for the layer is
   not null, we just free it. If we set the data pointer to the cipher, we'd
   call free on the cipher object without calling its destructor (it needs
   one) */ 
typedef struct {
  cipher_t *cipher;
} cipher_pointer_t;

static void symmetric_cipher_down(proto_stack_t *stack, slan_message_t 
				  *message) {
  cipher_t *cipher;

  cipher = ((cipher_pointer_t *) stack->data)->cipher;

  cipher->cbc_encrypt(cipher, message->data, message->length);
#ifdef DEBUG
  fprintf(stderr,"sym_cipher: Encrypted a packet.\n");
#endif
  
  stack->next->down(stack->next, message);
}

static void symmetric_cipher_up(proto_stack_t *stack, slan_message_t
				*message) {
  cipher_t *cipher;

  cipher = ((cipher_pointer_t *) stack->data)->cipher;

  cipher->cbc_decrypt(cipher, message->data, message->length);
#ifdef DEBUG
  fprintf(stderr,"sym_cipher: decrypted a packet.\n");
#endif

  stack->prev->up(stack->prev, message);
}

/* We assume that we will be passed a cipher instance that is already
   setup. It will be the responsiblity of the cipher user to call the
   cipher's key set function */
int symmetric_cipher_register(proto_stack_t *stack_entry, cipher_t
			      *cipher) {

  stack_entry->up = symmetric_cipher_up;
  stack_entry->down = symmetric_cipher_down;
  stack_entry->header_reserve = 0;
  stack_entry->trailer_reserve = 0;
  stack_entry->data = malloc(sizeof(cipher_pointer_t));
  ((cipher_pointer_t *)stack_entry->data)->cipher = cipher;

  return 0;
}

